# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

snake_case = 0x01
true_property = 0x02

all_feature_names = [
    "snake_case",
    "true_property",
]
